#!/usr/bin/perl
#
################################################################################
#
# IPCop Connection Scheduler Web-Iface
#
# Copyright (C) 2006-2009 Olaf (weizen_42) Westrik
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA 
#
#
# Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der GNU General Public License, wie von der Free Software Foundation verffentlicht, weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder (nach Ihrer Option) jeder spteren Version. 
#
# Die Verffentlichung dieses Programms erfolgt in der Hoffnung, da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 
#
# Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem Programm erhalten haben. Falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
#
################################################################################
#
# For support post / read in http://www.ipcop-forum.de
#
# $Id: connscheduler.cgi 348 2009-03-28 17:09:56Z weizen_42 $
#
# Revision history:
#
# 2006-04-xx created by weizen_42 at ipcop-forum dot de
# 2006-06-xx added reboot and shutdown
# 2006-08-20 added ipsecctrl
# 2006-10-08 added check for new version
# 2006-10-28 added openvpnctrl
# 2006-11-10 bugfixed up/down button with only 1 event defined
#            removed some warning messages in httpd/error.log
#
#


use strict;

# enable only the following on debugging purpose
use warnings;
use CGI::Carp 'fatalsToBrowser';

use IO::Socket;
use LWP::UserAgent;

require '/var/ipcop/general-functions.pl';
require '/var/ipcop/lang.pl';
require '/var/ipcop/header.pl';

require '/var/ipcop/connscheduler/lib.pl';


my $debug = 0;
# checking for new version
my $addonname = 'connscheduler';
my $onlineversion = '';
my $onlinelink = '';
my $timestamplastcheck = '/var/ipcop/connscheduler/lastcheck';
my $flagdonotcheck = '/var/ipcop/connscheduler/noversioncheck';

# get rid of used only once warnings
my @onlyonce = ( $Header::table1colour, $Header::table2colour, $Header::colouryellow, $CONNSCHED::configsize );
undef @onlyonce;

my $buttontext = $Lang::tr{'add'};
my $hiddenvalue = 'add';
my $day;
my $hour;
my $minute;
my %temppppsettings=();
my @profilenames=();
my @wol_devices = ();

#
# defaults for settings
#
my $selected_hour = '00';
my $selected_minute = '00';
my $checked_connect = "checked='checked'";
my $checked_profile = '';
my $checked_wakeonlan = '';
my %selected = ();
$selected{'reconnect'} = '';
$selected{'dial'} = '';
$selected{'hangup'} = '';
$selected{'reboot'} = '';
$selected{'shutdown'} = '';
$selected{'ovpnstart'} = '';
$selected{'ovpnstop'} = '';
$selected{'ipsecstart'} = '';
$selected{'ipsecstop'} = '';
$selected{'wlanapstart'} = '';
$selected{'wlanapstop'} = '';
my $selected_profile = 1;
my $selected_wakeonlan = '';
my $checked_days = "checked='checked'";
my $selected_daystart = 1;
my $selected_dayend = 31;
my $checked_weekdays = '';
my $checked_mon = "checked='checked'";
my $checked_tue = "checked='checked'";
my $checked_wed = "checked='checked'";
my $checked_thu = "checked='checked'";
my $checked_fri = "checked='checked'";
my $checked_sat = "checked='checked'";
my $checked_sun = "checked='checked'";
my $comment = '';

my %cgiparams = ();

$cgiparams{'ACTION'} = '';              # add/edit/update/remove/wakeup
$cgiparams{'ACTION_ACTION'} = '';       # CONNECT/PROFILE
$cgiparams{'ACTION_CONNECT'} = '';      # connect/disconnect/reconnect
$cgiparams{'ACTION_PROFILENR'} = 0;
$cgiparams{'ACTION_HOUR'} = '';
$cgiparams{'ACTION_MINUTE'} = '';
$cgiparams{'ACTION_DAYSTYPE'} = '';
$cgiparams{'ACTION_DAYSTART'} = 1;
$cgiparams{'ACTION_DAYEND'} = 31;
$cgiparams{'Mon'} = '';
$cgiparams{'Tue'} = '';
$cgiparams{'Wed'} = '';
$cgiparams{'Thu'} = '';
$cgiparams{'Fri'} = '';
$cgiparams{'Sat'} = '';
$cgiparams{'Sun'} = '';
$cgiparams{'ACTION_COMMENT'} = '';
$cgiparams{'ID'} = '';

&Header::getcgihash(\%cgiparams);


# read the profile names
my $i=0;
for ($i = 1; $i <= $CONNSCHED::maxprofiles; $i++)
{
  %temppppsettings = ();
  $temppppsettings{'PROFILENAME'} = $Lang::tr{'empty'};
  &General::readhash("/var/ipcop/ppp/settings-$i", \%temppppsettings);
  $profilenames[$i] = $temppppsettings{'PROFILENAME'};
}

&Header::showhttpheaders();
&Header::openpage($Lang::tr{'connscheduler'}, 1, '');
&Header::openbigbox('100%', 'left', '', '');


# Found this usefull piece of code in BlockOutTraffic AddOn  8-)
#   fwrules.cgi
###############
# DEBUG DEBUG
if ( $debug )
{
&Header::openbox('100%', 'left', 'DEBUG');
my $debugCount = 0;
foreach my $line (sort keys %cgiparams) {
  print "$line = $cgiparams{$line}<br />\n";
  $debugCount++;
}
print "&nbsp;Count: $debugCount\n";
&Header::closebox();
}
# DEBUG DEBUG
###############


#
# Check for new version
#
&checkfornewversion($addonname, $CONNSCHED::version);
if ( $onlineversion ne '' )
{
  &Header::openbox('100%', 'left', $Lang::tr{'info'});
  print <<END
<table width="100%"><tr>
<td>$Lang::tr{'ConnSched newversion'} <a href="$onlinelink" target="_blank"><b>$onlineversion</b></a></td>
</tr></table>
END
;
  &Header::closebox();
}


if ( $cgiparams{'ACTION'} eq 'toggle' )
{
  if ( $CONNSCHED::config[$cgiparams{'ID'}]{'ACTIVE'} eq 'on' )
  {
    $CONNSCHED::config[$cgiparams{'ID'}]{'ACTIVE'} = 'off';
  }
  else
  {
    $CONNSCHED::config[$cgiparams{'ID'}]{'ACTIVE'} = 'on';
  }
  
  &CONNSCHED::WriteConfig;
}
elsif ( ($cgiparams{'ACTION'} eq 'add') || ($cgiparams{'ACTION'} eq 'update') )
{
  my $l_action = $cgiparams{'ACTION_CONNECT'};
  my $l_profilenr = '';
  my $l_days = '';
  my $l_weekdays = '';

  if ( $cgiparams{'ACTION'} eq 'add' )
  {
    $i = $#CONNSCHED::config + 1;
    $CONNSCHED::config[$i]{'ACTIVE'} = 'on';
  }
  else
  {
    $i = $cgiparams{'UPDATE_ID'};
  }

  if ( $cgiparams{'ACTION_ACTION'} eq 'PROFILE' )
  {
    $l_action = 'select profile';
    $l_profilenr = $cgiparams{'ACTION_PROFILENR'};
  }
  elsif ( $cgiparams{'ACTION_ACTION'} eq 'WAKEONLAN' )
  {
    $l_action = 'wol';
    $l_profilenr = $cgiparams{'ACTION_WOL'};
  }

  if ( $cgiparams{'ACTION_DAYSTYPE'} eq 'WEEKDAYS' )
  {
    if ( $cgiparams{'Mon'} eq 'on' ) { $l_weekdays .= 'Mon '; }
    if ( $cgiparams{'Tue'} eq 'on' ) { $l_weekdays .= 'Tue '; }
    if ( $cgiparams{'Wed'} eq 'on' ) { $l_weekdays .= 'Wed '; }
    if ( $cgiparams{'Thu'} eq 'on' ) { $l_weekdays .= 'Thu '; }
    if ( $cgiparams{'Fri'} eq 'on' ) { $l_weekdays .= 'Fri '; }
    if ( $cgiparams{'Sat'} eq 'on' ) { $l_weekdays .= 'Sat '; }
    if ( $cgiparams{'Sun'} eq 'on' ) { $l_weekdays .= 'Sun '; }
  }
  else
  {
    $l_days = "$cgiparams{'ACTION_DAYSTART'} - $cgiparams{'ACTION_DAYEND'}";
  }

  $CONNSCHED::config[$i]{'ACTION'} = $l_action;
  $CONNSCHED::config[$i]{'PROFILENR'} = $l_profilenr;
  $CONNSCHED::config[$i]{'TIME'} = "$cgiparams{'ACTION_HOUR'}:$cgiparams{'ACTION_MINUTE'}";
  $CONNSCHED::config[$i]{'DAYSTYPE'} = lc($cgiparams{'ACTION_DAYSTYPE'});
  $CONNSCHED::config[$i]{'DAYS'} = $l_days;
  $CONNSCHED::config[$i]{'WEEKDAYS'} = $l_weekdays;
  $CONNSCHED::config[$i]{'COMMENT'} = &Header::cleanhtml($cgiparams{'ACTION_COMMENT'});

  &CONNSCHED::WriteConfig;
}
elsif ( $cgiparams{'ACTION'} eq 'edit' )
{
  $i = $cgiparams{'ID'};
  
  $selected_hour = substr($CONNSCHED::config[$i]{'TIME'},0,2);
  $selected_minute = substr($CONNSCHED::config[$i]{'TIME'},3,2);

  $checked_connect = '';
  $checked_profile = '';
  $checked_wakeonlan = '';
  if ( $CONNSCHED::config[$i]{'ACTION'} eq 'select profile' )
  {
    $checked_profile = "checked='checked'";
    $selected_profile = $CONNSCHED::config[$i]{'PROFILENR'};
  }
  elsif ( $CONNSCHED::config[$i]{'ACTION'} eq 'wol' )
  {
    $checked_wakeonlan = "checked='checked'";
    $selected_wakeonlan = $CONNSCHED::config[$i]{'PROFILENR'};
  }
  else
  {
    $checked_connect = "checked='checked'";
    $selected{"$CONNSCHED::config[$i]{'ACTION'}"} = "selected='selected'";
  }

  if ( $CONNSCHED::config[$i]{'DAYSTYPE'} eq 'days' )
  {
    my @temp = split(/-/,$CONNSCHED::config[$i]{'DAYS'},2);

    $selected_daystart = substr($temp[0], 0, -1);
    $selected_dayend = substr($temp[1], 1);
  }
  else
  {
    my $wd = $CONNSCHED::config[$i]{'WEEKDAYS'};
    $checked_mon = '' if ( index($wd, 'Mon') == -1 ) ;
    $checked_tue = '' if ( index($wd, 'Tue') == -1 ) ;
    $checked_wed = '' if ( index($wd, 'Wed') == -1 ) ;
    $checked_thu = '' if ( index($wd, 'Thu') == -1 ) ;
    $checked_fri = '' if ( index($wd, 'Fri') == -1 ) ;
    $checked_sat = '' if ( index($wd, 'Sat') == -1 ) ;
    $checked_sun = '' if ( index($wd, 'Sun') == -1 ) ;

    $checked_days = '';
    $checked_weekdays = "checked='checked'";
  }

  $comment = $CONNSCHED::config[$cgiparams{'ID'}]{'COMMENT'};

  $buttontext = $Lang::tr{'update'};
  $hiddenvalue = 'update';
}
elsif ( $cgiparams{'ACTION'} eq 'remove' )
{
  # simply set ACTIVE to empty, WriteConfig will handle the gory details
  $CONNSCHED::config[$cgiparams{'ID'}]{'ACTIVE'} = '';
  &CONNSCHED::WriteConfig;
}
elsif ( ($CONNSCHED::configsize >= 2) && (($cgiparams{'ACTION'} eq 'down') || ($cgiparams{'ACTION'} eq 'up')) )
{
  my $action = @CONNSCHED::config[$cgiparams{'ID'}];
  my $newpos = 0;

  splice(@CONNSCHED::config, $cgiparams{'ID'}, 1);

  if ( ($cgiparams{'ACTION'} eq 'down') )
  {
    $newpos = $cgiparams{'ID'} + 1;
  }
  else
  {
    $newpos = $cgiparams{'ID'} - 1;
  }

  splice(@CONNSCHED::config, $newpos, 0, $action);

  &CONNSCHED::WriteConfig;
}


#
# Add / Edit Box
#

&Header::openbox('100%', 'left', $Lang::tr{'ConnSched add action'});

print <<END
<form method='post' name='addevent' action='$ENV{'SCRIPT_NAME'}'>
<table width='100%' border='0' cellspacing='6' cellpadding='0'>
<tr>
<td width='15%' class='base'>$Lang::tr{'ConnSched time'}</td>
<td><select name='ACTION_HOUR'>
END
;
for ($hour = 0; $hour <= 23; $hour++)
{
  my $hour00 = $hour < 10 ? "0$hour" : $hour;
  if ( $hour00 eq $selected_hour )
  {
    print "<option value='$hour00' selected='selected'>$hour00</option>";
  }
  else
  {
    print "<option value='$hour00'>$hour00</option>";
  }
}
print "</select>&nbsp;:&nbsp;<select name='ACTION_MINUTE'>";
for ($minute = 0; $minute <= 55; $minute += 5)
{
  my $minute00 = $minute < 10 ? "0$minute" : $minute;
  if ( $minute00 eq $selected_minute )
  {
    print "<option value='$minute00' selected='selected'>$minute00</option>";
  }
  else
  {
    print "<option value='$minute00'>$minute00</option>";
  }
}

print <<END
</select></td></tr>
<tr><td colspan='2'><hr /></td></tr>
<tr><td width='15%' class='base'>$Lang::tr{'ConnSched action'}</td><td>
<input type='radio' value='CONNECT' name='ACTION_ACTION' $checked_connect />&nbsp;<select name='ACTION_CONNECT'>
<option value='reconnect' $selected{'reconnect'}>$Lang::tr{'ConnSched reconnect'}</option>
<option value='dial' $selected{'dial'}>$Lang::tr{'dial'}</option>
<option value='hangup' $selected{'hangup'}>$Lang::tr{'hangup'}</option>
<option value='reboot' $selected{'reboot'}>$Lang::tr{'reboot'}</option>
<option value='shutdown' $selected{'shutdown'}>$Lang::tr{'shutdown'}</option>
<option value='ovpnstart' $selected{'ovpnstart'}>$Lang::tr{'ConnSched ovpnstart'}</option>
<option value='ovpnstop' $selected{'ovpnstop'}>$Lang::tr{'ConnSched ovpnstop'}</option>
<option value='ipsecstart' $selected{'ipsecstart'}>$Lang::tr{'ConnSched ipsecstart'}</option>
<option value='ipsecstop' $selected{'ipsecstop'}>$Lang::tr{'ConnSched ipsecstop'}</option>
<option value='wlanapstart' $selected{'wlanapstart'}>$Lang::tr{'ConnSched wlanapstart'}</option>
<option value='wlanapstop' $selected{'wlanapstop'}>$Lang::tr{'ConnSched wlanapstop'}</option>
</select></td></tr>
<tr><td width='15%' class='base'>&nbsp;</td>
<td><input type='radio' value='PROFILE' name='ACTION_ACTION' $checked_profile />&nbsp;$Lang::tr{'ConnSched change profile title'}&nbsp;<select name='ACTION_PROFILENR'>
END
;
for ($i = 1; $i <= $CONNSCHED::maxprofiles; $i++)
{
  my $selected_opt = '';
  $selected_opt = "selected='selected'" if ( $i == $selected_profile );
  print "<option value='$i' $selected_opt>$i. $profilenames[$i]</option>";
}
print "</select></td></tr>";

if ( -e "/var/ipcop/wakeonlan-gui/clients.conf" )
{
  &ReadConfig_WOL;
  print "<tr><td width='15%' class='base'>&nbsp;</td>";
  print "<td><input type='radio' value='WAKEONLAN' name='ACTION_ACTION' $checked_wakeonlan />&nbsp;$Lang::tr{'ConnSched WOL title'}&nbsp;<select name='ACTION_WOL'>";
  for $i ( 0 .. $#wol_devices )
  {
    my $selected_opt = '';
    $selected_opt = "selected='selected'" if ( $wol_devices[$i]{'IFace'}." ".$wol_devices[$i]{'MAC'} eq $selected_wakeonlan );
    my $wol_txt = $Lang::tr{$wol_devices[$i]{'IFace'}}." ".$wol_devices[$i]{'MAC'}." ".&Header::cleanhtml($wol_devices[$i]{'Comment'});
    print "<option value='$wol_devices[$i]{'IFace'} $wol_devices[$i]{'MAC'}' $selected_opt>$wol_txt</option>";
  }
  print "</select></td></tr>";
}

print <<END
<tr><td colspan='2'><hr /></td></tr>
<tr><td width='15%' class='base'>$Lang::tr{'ConnSched days'}</td>
<td><input type='radio' value='DAYS' name='ACTION_DAYSTYPE' $checked_days />&nbsp;<select name='ACTION_DAYSTART'>
END
;
for ($day = 1; $day <= 31; $day++)
{
  if ( $day == $selected_daystart )
  {
    print "<option value='$day' selected='selected'>$day</option>";
  }
  else
  {
    print "<option value='$day'>$day</option>";
  }
}
print "</select>&nbsp;-&nbsp;<select name='ACTION_DAYEND'>";
for ($day = 1; $day <= 31; $day++)
{
  if ( $day == $selected_dayend )
  {
    print "<option value='$day' selected='selected'>$day</option>";
  }
  else
  {
    print "<option value='$day'>$day</option>";
  }
}

print <<END
</select></td></tr>
<tr><td width='15%' class='base'>&nbsp;</td><td><input type='radio' value='WEEKDAYS' name='ACTION_DAYSTYPE' $checked_weekdays />&nbsp;$Lang::tr{'ConnSched weekdays'}<br />
&nbsp;&nbsp;<input type='checkbox' name='Mon' $checked_mon />$Lang::tr{'monday'}<br />
&nbsp;&nbsp;<input type='checkbox' name='Tue' $checked_tue />$Lang::tr{'tuesday'}<br />
&nbsp;&nbsp;<input type='checkbox' name='Wed' $checked_wed />$Lang::tr{'wednesday'}<br />
&nbsp;&nbsp;<input type='checkbox' name='Thu' $checked_thu />$Lang::tr{'thursday'}<br />
&nbsp;&nbsp;<input type='checkbox' name='Fri' $checked_fri />$Lang::tr{'friday'}<br />
&nbsp;&nbsp;<input type='checkbox' name='Sat' $checked_sat />$Lang::tr{'saturday'}<br />
&nbsp;&nbsp;<input type='checkbox' name='Sun' $checked_sun />$Lang::tr{'sunday'}
</td></tr>
<tr><td colspan='2'><hr /></td></tr>
<tr><td width='15%' class='base'>$Lang::tr{'remark title'}&nbsp;<img src='/blob.gif' alt='*' /></td>
<td><input type='text' name='ACTION_COMMENT' size='40' value='$comment' /></td></tr></table>
<hr />
<table width='100%'><tr>
  <td class='base' valign='top'><img src='/blob.gif' alt='*' /></td>
  <td width='55%' class='base'>$Lang::tr{'this field may be blank'}</td>
  <td width='40%' align='center'><input type='submit' name='SUBMIT' value='$buttontext' />
  <input type='hidden' name='ACTION' value='$hiddenvalue' /></td>
  <input type='hidden' name='UPDATE_ID' value='$cgiparams{'ID'}' /></td>
</tr></table>
</form>

END
;

&Header::closebox();

#
# Box with List of events
#

&Header::openbox('100%', 'left', $Lang::tr{'ConnSched scheduled actions'});
print <<END
<table width='100%' border='0' cellspacing='1' cellpadding='0'>
<tr>
<td align='center' width='10%'><b>$Lang::tr{'time'}</b></td>
<td width='15%'>&nbsp;</td>
<td align='center' width='60%'><b>$Lang::tr{'remark'}</b></td>
<td align='center' colspan='5'><b>$Lang::tr{'action'}</b></td>
</tr>
END
;

for my $id ( 0 .. $#CONNSCHED::config )
{
  my $trcolor;

  if ( ($cgiparams{'ACTION'} eq 'edit') && ($id == $cgiparams{'ID'}) ) 
  {
    $trcolor = "<tr bgcolor='${Header::colouryellow}'>";
  }
  elsif ( $id % 2 )
  {
    $trcolor = "<tr bgcolor='$Header::table2colour'>";
  }
  else 
  {
    $trcolor = "<tr bgcolor='$Header::table1colour'>";
  }

print <<END
$trcolor
<td align='center'>$CONNSCHED::config[$id]{'TIME'}</td>
<td>$Lang::tr{"ConnSched $CONNSCHED::config[$id]{'ACTION'}"}&nbsp;$CONNSCHED::config[$id]{'PROFILENR'}</td>
<td>$CONNSCHED::config[$id]{'COMMENT'}</td>
<td align='center'>
  <form method='post' name='frm$id' action='$ENV{'SCRIPT_NAME'}'>
  <input type='hidden' name='ACTION' value='toggle' />
  <input type='image' name='$Lang::tr{'toggle enable disable'}' src='/images/$CONNSCHED::config[$id]{'ACTIVE'}.gif' alt='$Lang::tr{'toggle enable disable'}' title='$Lang::tr{'toggle enable disable'}' />
  <input type='hidden' name='ID' value='$id' />
  </form>
</td>
<td align='center'>
  <form method='post' name='frm$id' action='$ENV{'SCRIPT_NAME'}'>
  <input type='hidden' name='ACTION' value='edit' />
  <input type='image' name='$Lang::tr{'edit'}' src='/images/edit.gif' alt='$Lang::tr{'edit'}' title='$Lang::tr{'edit'}' />
  <input type='hidden' name='ID' value='$id' />
  </form>
</td>
<td align='center'>
  <form method='post' name='frm$id' action='$ENV{'SCRIPT_NAME'}'>
  <input type='hidden' name='ACTION' value='remove' />
  <input type='image' name='$Lang::tr{'remove'}' src='/images/delete.gif' alt='$Lang::tr{'remove'}' title='$Lang::tr{'remove'}' />
  <input type='hidden' name='ID' value='$id' />
  </form>
</td>
<td align='center'>
  <form method='post' name='frm$id' action='$ENV{'SCRIPT_NAME'}'>
  <input type='hidden' name='ACTION' value='up' />
  <input type='image' name='$Lang::tr{'ConnSched up'}' src='/images/up.gif' alt='$Lang::tr{'ConnSched up'}' title='$Lang::tr{'ConnSched up'}' />
  <input type='hidden' name='ID' value='$id' />
  </form>
</td>
<td align='center'>
  <form method='post' name='frm$id' action='$ENV{'SCRIPT_NAME'}'>
  <input type='hidden' name='ACTION' value='down' />
  <input type='image' name='$Lang::tr{'ConnSched down'}' src='/images/down.gif' alt='$Lang::tr{'ConnSched down'}' title='$Lang::tr{'ConnSched down'}' />
  <input type='hidden' name='ID' value='$id' />
  </form>
</td>
</tr>
$trcolor
<td>&nbsp;</td>
<td colspan='7'>$CONNSCHED::config[$id]{'DAYS'}$CONNSCHED::config[$id]{'WEEKDAYS_PR'}&nbsp;</td>
</tr>
END
;
}

print <<END
</table>
<br />
<hr />
<table width='100%'>
<tr>
	<td>&nbsp;</td>
	<td align='right'>
	<b><small><a href="http://www.ban-solms.de/t/IPCop.html" target="_blank">Connection Scheduler $CONNSCHED::version</a></small></b>
	</td>
</tr>
</table>
END
;


&Header::closebox();

&Header::closebigbox();
&Header::closepage();


#
# load the WOL configuration file
#
sub ReadConfig_WOL
{
  # datafileformat:
  #   ID,MAC,IFACE,,Comment
  # 
  my @tmpfile = ();
  if ( open(FILE, "/var/ipcop/wakeonlan-gui/clients.conf") )
  {
    @tmpfile = <FILE>;
    close (FILE);
  }

  @wol_devices = ();

  # populate devices list
  foreach my $line ( @tmpfile )
  {
    chomp($line);               # remove newline
    my @temp = split(/\,/,$line,5);
    if ( $temp[1] eq '' ) { next; }
    unless(&General::validmac($temp[1])) { next; }

    push @wol_devices, { ID => $temp[0], MAC => $temp[1], IFace => $temp[2], Comment => $temp[4] };
  }
}


sub checkfornewversion
{
  my $addon = shift;
  my $version = shift;
  $onlineversion = '';

  if ( -e $flagdonotcheck )
  {
    return;
  }

  # only check if we are online and last check was some time ago
  if ( (! -e '/var/ipcop/red/active') || (-e $timestamplastcheck) && (int(-M $timestamplastcheck) < 5) )
  {
    return;
  }

#workaround to suppress a warning when a variable is used only once
  my @dummy = ( $General::version );
  undef (@dummy);

  my $ua = LWP::UserAgent->new;
  $ua->timeout(120);
  $ua->agent("Mozilla/4.0 (compatible; IPCop $General::version; $version)");
  my $content = $ua->get("http://ipcop-addons.ath.cx/version/$addon");

  if ( $content->is_success )
  {
    # compare the versions, format is v1.2.3
    $content->content =~ /v(\d+).(\d+).(\d+)/;
    my $ver1 = $1;
    my $ver2 = $2;
    my $ver3 = $3;

    $version =~ /v(\d+).(\d+).(\d+)/;

    if ( ($ver1 > $1) || (($ver1 == $1) && ($ver2 > $2)) || (($ver1 == $1) && ($ver2 == $2) && ($ver3 > $3)) )
    {
      $onlineversion = "v$ver1.$ver2.$ver3";

      $content->content =~ /http(.*)/;
      $onlinelink = "http$1";
    }
    else
    {
      # no news, recheck in a couple of days
      system("touch $timestamplastcheck");
    }
  }
}
