#!/usr/bin/perl
#
################################################################################
#
# IPCop Connection Scheduler (F)Cron Job
#
# Copyright (C) 2006-2009 Olaf (weizen_42) Westrik
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA 
#
#
# Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der GNU General Public License, wie von der Free Software Foundation verffentlicht, weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder (nach Ihrer Option) jeder spteren Version. 
#
# Die Verffentlichung dieses Programms erfolgt in der Hoffnung, da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 
#
# Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem Programm erhalten haben. Falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
#
################################################################################
#
# For support post / read in http://www.ipcop-forum.de
#
# $Id: connscheduler.pl 348 2009-03-28 17:09:56Z weizen_42 $
#
# Revision history:
#
# 2006-04-xx created by weizen_42 at ipcop-forum dot de
# 2006-06-xx added reboot and shutdown
# 2006-08-20 added ipsecctrl
# 2006-10-08 rewrite ppp secrets when changing profile
# 2006-10-28 added openvpnctrl
#

use strict;

require '/var/ipcop/general-functions.pl';
require '/var/ipcop/connscheduler/lib.pl';


# seems to be necessary
my $sleep_after_profile = 5;

my ($second, $minute, $hour, $day, $month ,$year, $weekday) = localtime(time);
# correction for weekday, I am used to weeks starting with Monday (= 0) ;-)
$weekday = ($weekday + 6) % 7;
# get the closest thing possible
$minute = int($minute / 5) * 5;


if ( $ARGV[0] eq 'hangup' )
{
  &hangup();
}
elsif ( $ARGV[0] eq 'dial' )
{
  &dial();
}
elsif ( $ARGV[0] eq 'reconnect' )
{
  &reconnect();
}
elsif ( $ARGV[0] eq 'profile' )
{
  &profile($ARGV[1]);
}
elsif ( $ARGV[0] eq 'timer' )
{
  &timer();
}
elsif ( $ARGV[0] eq 'test' )
{
  &test();
}
else
{
  print "Usage: $0 {dial | hangup | reconnect | profile nr# }\n";
}

exit 0;


#   __                  _   _
#  / _|                | | (_)
# | |_ _   _ _ __   ___| |_ _  ___  _ __  ___
# |  _| | | | '_ \ / __| __| |/ _ \| '_ \/ __|
# | | | |_| | | | | (__| |_| | (_) | | | \__ \
# |_|  \__,_|_| |_|\___|\__|_|\___/|_| |_|___/
#
sub hangup
{
  &General::log("ConnSched disconnect");

  unless ( -e "${General::swroot}/red/active" )
  {
    &General::log("ConnSched already disconnected");
    return;
  }
  unless ( system('/etc/rc.d/rc.red', 'stop') == 0 )
  {
    &General::log("ConnSched disconnect failed: $?");
    return;
  }

  # now wait for active triggerfile and ppp daemon to disappear 
  sleep 1;
  while ( -e "${General::swroot}/red/active" || -e '/var/run/ppp-ipcop.pid' ) 
  {
    sleep 1;
  }
}


sub dial
{
  &General::log("ConnSched connect");

  if ( -e "${General::swroot}/red/active" )
  {
    &General::log("ConnSched already connected");
    return;
  }
  unless ( system('/etc/rc.d/rc.red', 'start') == 0 )
  {
    &General::log("ConnSched connect failed: $?");
    return;
  }

  # wait maximum 60 seconds for active triggerfile
  my $counter = 60;
  until ( -e "${General::swroot}/red/active" || $counter == 0 )
  {
    sleep 1;
    $counter--;
  }
}


sub reconnect
{
  &General::log("ConnSched reconnect");
  &hangup() if ( -e "${General::swroot}/red/active" );
  &dial();
}


sub profile
{
  my $profile = shift;
  my $restart_red = 0;

  unless ( ($profile > 0) and ($profile < $CONNSCHED::maxprofiles) )
  {
    &General::log("ConnSched invalid profile: $profile");
    return;
  }

  unless ( -e "${General::swroot}/ppp/settings-$profile" )
  {
    &General::log("ConnSched profile file does not exist: $profile");
    return;
  }

  if ( -e "${General::swroot}/red/active" )
  {
    # remember to restart red after changing profile
    $restart_red = 1;
    &hangup();
  }

  &General::log("ConnSched select profile: $profile");

  # Method to change Profile from pppsetup.cgi
	unlink("${General::swroot}/ppp/settings");
	link("${General::swroot}/ppp/settings-$profile", "${General::swroot}/ppp/settings");
	system("/bin/touch", "${General::swroot}/ppp/updatesettings");

  # change secrets (see SF bug 1557321)
  my %temppppsettings=();
  &General::readhash("${General::swroot}/ppp/settings", \%temppppsettings);
  system("/bin/echo \"'$temppppsettings{'USERNAME'}' * '$temppppsettings{'PASSWORD'}'\" > ${General::swroot}/ppp/secrets");

  if ( $restart_red == 1 )
  {
    ## FIXME: do we need to do this ?
    sleep($sleep_after_profile);
    &dial();
  }  
}


# fcronjob entry
sub timer
{
  for my $i ( 0 .. $#CONNSCHED::config )
  {
    next if ( $CONNSCHED::config[$i]{'ACTIVE'} ne 'on' );

    my $action_hour = substr($CONNSCHED::config[$i]{'TIME'},0,2);
    my $action_minute = substr($CONNSCHED::config[$i]{'TIME'},3,2);

    next if ( $action_hour != $hour );
    next if ( $action_minute != $minute );

    if ( $CONNSCHED::config[$i]{'DAYSTYPE'} eq 'days' )
    {
      my @temp = split(/-/,$CONNSCHED::config[$i]{'DAYS'},2);

      my $daystart = substr($temp[0], 0, -1);
      my $dayend = substr($temp[1], 1);

      next if ( ($day < $daystart) || ($day > $dayend) );
    }
    else
    {
      next if ( index($CONNSCHED::config[$i]{'WEEKDAYS'}, $CONNSCHED::weekdays[$weekday]) == -1 );
    }


    if ( $CONNSCHED::config[$i]{'ACTION'} eq 'reconnect' )    
    {
      &reconnect()
    }
    elsif ( $CONNSCHED::config[$i]{'ACTION'} eq 'dial' )
    {
      &dial();
    }
    elsif ( $CONNSCHED::config[$i]{'ACTION'} eq 'hangup' )
    {
      &hangup();
    }
    elsif ( $CONNSCHED::config[$i]{'ACTION'} eq 'select profile' )
    {
      &profile($CONNSCHED::config[$i]{'PROFILENR'});
    }
    elsif ( $CONNSCHED::config[$i]{'ACTION'} eq 'reboot' )
    {
      &General::log("ConnSched reboot");
      system ("/usr/local/bin/ipcopreboot", "boot");
    }
    elsif ( $CONNSCHED::config[$i]{'ACTION'} eq 'shutdown' )
    {
      &General::log("ConnSched shutdown");
      system ("/usr/local/bin/ipcopreboot", "down");
    }
    elsif ( $CONNSCHED::config[$i]{'ACTION'} eq 'ovpnstart' )
    {
      &General::log("ConnSched OpenVPN start");
      system ("/usr/local/bin/openvpnctrl", "--restart");
    }
    elsif ( $CONNSCHED::config[$i]{'ACTION'} eq 'ovpnstop' )
    {
      &General::log("ConnSched OpenVPN stop");
      system ("/usr/local/bin/openvpnctrl", "--kill");
      system ("/usr/local/bin/openvpnctrl", "--delete-chains-and-rules");
    }
    elsif ( $CONNSCHED::config[$i]{'ACTION'} eq 'ipsecstart' )
    {
      &General::log("ConnSched ipsecstart");
      system ("/usr/local/bin/ipsecctrl", "S");
    }
    elsif ( $CONNSCHED::config[$i]{'ACTION'} eq 'ipsecstop' )
    {
      &General::log("ConnSched ipsecstop");
      system ("/usr/local/bin/ipsecctrl", "D");
    }
    elsif ( $CONNSCHED::config[$i]{'ACTION'} eq 'wlanapstart' )
    {
      &General::log("ConnSched wlanapstart");
      system ("/etc/rc.d/rc.hostapd", "start") if ( -e "/etc/rc.d/rc.hostapd" );
      system ("/etc/rc.d/rc.wlanap", "start") if ( -e "/etc/rc.d/rc.wlanap" );
    }
    elsif ( $CONNSCHED::config[$i]{'ACTION'} eq 'wlanapstop' )
    {
      &General::log("ConnSched wlanapstop");
      system ("/etc/rc.d/rc.hostapd", "stop") if ( -e "/etc/rc.d/rc.hostapd" );
      system ("/etc/rc.d/rc.wlanap", "stop") if ( -e "/etc/rc.d/rc.wlanap" );
    }
    elsif ( $CONNSCHED::config[$i]{'ACTION'} eq 'wol' )
    {
      &General::log("ConnSched WOL $CONNSCHED::config[$i]{'PROFILENR'}");
      my @temp = split(/ /,$CONNSCHED::config[$i]{'PROFILENR'},2);
      my $mac = $temp[1];
      my $iface = uc($temp[0]).'_DEV';
      &General::log("ConnSched WOL $mac $iface");
      my %netsettings = ();
      &General::readhash("${General::swroot}/ethernet/settings", \%netsettings);
      $iface = $netsettings{"$iface"};
      &General::log("ConnSched WOL $mac $iface");
      system("/usr/local/bin/launch-ether-wake $mac $iface");
    }
    else
    {
      # okay ? an event we don't know about
      #&General::log("ConnSched unknown event $CONNSCHED::config[$i]{'ACTION'}");
    }
  }
}
